import template from './config-amazon-editor.tmpl.html';

export default function configAmazonEditor($log, $q, $rootScope, $window, ConvoworksApi, LoginService, AlertService) {
    return {
        restrict: 'E',
        scope: { service: '=' },
        template: template,
        controller: function ($scope) {

        },
        link: function ($scope, $element, $attributes) {
            var config_options = {};
            var user    =   null;

            LoginService.getUser().then( function ( u) {
                user = u;
            });

            $scope.config = {
                mode: 'manual',
                invocation: $scope.service.name,
                app_id: null,
                default_locale: 'en-US',
                interaction_model_sensitivity: 'LOW',
                supported_locales: ['en-US'],
                propagate_to_all_english: false,
                auto_display: false
            };

            var configBak   =   angular.copy( $scope.config);
            var is_new      =   true;
            var is_error    =   false;

            _load();

            $scope.gotoConfigUrl = function() {
                $window.open('https://developer.amazon.com/alexa/console/ask/publish/alexapublishing/' + $scope.config.app_id + '/development/en_US/skill-info', '_blank');
            }

            $scope.isModeValid  = function () {
                return !( $scope.config.mode === 'auto' && !user.amazon_account_linked);
            }

            $scope.isNew    = function () {
                return is_new;
            }

            $scope.updateConfig = function (isValid) {
                if (!isValid) {
                    throw new Error(`Invalid form data.`)
                }
                $log.debug('configAmazonEditor update() $scope.config', $scope.config);
                _updateSelectedInterfaces();
                if ( is_new) {
                    ConvoworksApi.createServicePlatformConfig( $scope.service.service_id, 'amazon', $scope.config).then(function (data) {
                        configBak = angular.copy( $scope.config);
                        is_new      =   false;
                        is_error    =   false;
                        AlertService.addSucess(`Service ${$scope.service.service_id} was linked successfully with Alexa.`);
                        $rootScope.$broadcast('ServiceConfigUpdated', $scope.config);
                    }, function ( response) {
                        $log.debug('configAmazonEditor create() response', response);
                        is_error    =   true;
                        throw new Error("Can't create config for Amazon. " + response.data.message)
                    });
                } else {
                    ConvoworksApi.updateServicePlatformConfig( $scope.service.service_id, 'amazon', $scope.config).then(function (data) {
                        configBak = angular.copy( $scope.config);
                        is_error    =   false;
                        $rootScope.$broadcast('ServiceConfigUpdated', $scope.config);
                    }, function ( response) {
                        $log.debug('configAmazonEditor update() response', response);
                        is_error    =   true;
                    });
                }
            }



            $scope.revertConfig = function () {
                $scope.config = angular.copy(configBak);
            }


            $scope.isConfigChanged = function () {
                return !angular.equals( configBak, $scope.config);
            }

            $scope.getAlexaInterfaces = function () {
                // update array with values from config
                if ($scope.config.interfaces && $scope.config.interfaces.length > 0) {
                    for (var i = 0; i < $scope.interfaces.length; i++) {
                        if ($scope.config.interfaces.includes($scope.interfaces[i].type)) {
                            $scope.interfaces[i].checked = true;
                        }
                    }
                }

                return $scope.interfaces;
            };

            function _updateSelectedInterfaces() {
                $scope.config.interfaces = [];
                for (var i = 0; i < $scope.interfaces.length; i++) {
                    if ($scope.interfaces[i].checked) {
                        var interfaceType = $scope.interfaces[i].type;
                        $scope.config.interfaces.push(interfaceType);
                    }
                }
            }

            function _arrayRemove(arr, value) {
                return arr.filter(function(ele) {
                    return ele !== value;
                });
            }

            $scope.registerChange = function(itfRecord) {
                var eventName = itfRecord.itf.type;
                var isInterfaceEnabled = !itfRecord.itf.checked;
                var newArr = [];

                if (isInterfaceEnabled) {
                    if ($scope.config.interfaces === undefined) {
                        newArr.push(eventName);
                        $scope.config.interfaces = newArr;
                    } else {
                        $scope.config.interfaces.push(eventName);
                    }
                } else {
                    if ($scope.config.interfaces === undefined) {
                        $scope.config.interfaces = _arrayRemove(newArr, eventName);
                    } else {
                        $scope.config.interfaces = _arrayRemove($scope.config.interfaces, eventName);
                    }
                }
                $scope.getAlexaInterfaces();
            }

            function _load()
            {
                ConvoworksApi.getConfigOptions().then(function (options) {
                    config_options = options;

                    $scope.languages = config_options['CONVO_AMAZON_LANGUAGES'];
                    $scope.sensitivities = config_options['CONVO_AMAZON_INTERACTION_MODEL_SENSITIVITIES'];
                    $scope.interfaces = config_options['CONVO_ALEXA_INTERFACES'];

                    $scope.$watch('config.auto_display', function(newVal) {
                        if (newVal !== undefined)
                        {
                            // $log.log('configAmazonEditor $watch config.auto_display new value', $scope.config);
                            $scope.config.auto_display = newVal;
                        }
                    });

                    $scope.$watch('config.propagate_to_all_english', function(isAllEnglish) {
                        if (isAllEnglish !== undefined)
                        {
                            $scope.config.propagate_to_all_english = isAllEnglish;

                            if (isAllEnglish) {
                                $scope.config.supported_locales = config_options['CONVO_AMAZON_ALL_ENGLISH'];
                            } else {
                                $scope.config.supported_locales = [$scope.config.default_locale]
                            }
                        }
                    });

                    $scope.$watch('config.default_locale', function(newDefaultLocale) {
                        if (newDefaultLocale !== undefined) {
                            // $log.log('configAmazonEditor $watch config.default_locale new value', $scope.config);
                            $scope.config.default_locale = newDefaultLocale;
                            if (!$scope.config.propagate_to_all_english) {
                                $scope.config.supported_locales = [$scope.config.default_locale]
                            }
                        }
                    });

                    $scope.$watch('config.interaction_model_sensitivity', function(newInteractionModelSensitivity) {
                        if (newInteractionModelSensitivity !== undefined) {
                            $scope.config.interaction_model_sensitivity = newInteractionModelSensitivity;
                        }
                    });

                    ConvoworksApi.getServicePlatformConfig( $scope.service.service_id, 'amazon').then(function (data) {
                        $scope.config = data;
                        configBak = angular.copy( $scope.config);
                        is_new  =   false;
                        is_error    =   false;
                    }, function ( response) {
                        $log.debug('configAmazonEditor loadPlatformConfig() response', response);

                        if ( response.status === 404) {
                            is_new      =   true
                            is_error    =   false;
                            return;
                        }
                        is_error    =   true;
                    });
                })
            }


        }
    }
}
