import template from './preview-panel.tmpl.html';

export default function previewPanel($log, $sce, ConvoworksApi, AlertService) {
    return {
        restrict: 'E',
        scope: {
            service: '=',
            blockId: '=?'
        },
        require: '^propertiesContext',
        template: template,
        link: function ($scope, $element, $attributes) {
            $log.log('previewPanel link');

            let previewBlocks = [];

            $scope.ready = false;
            $scope.filtered = [];

            $scope.search = { query: '' };

            $scope.generateText = function (text) {
                const content = text.map(t => t.text).join(' ');
                const markup = "<speak><p>" + content + "</p></speak>";

                _copyToClipboard(markup);
                AlertService.addInfo("Copied [" + markup + "]" + " to clipboard.");
            };

            _init();

            $scope.getUserMessageGroups = function(messages)
            {
                var found = [];
                var groups = [];

                for (var i in messages)
                {
                    if (!found.includes(messages[i].intent))
                    {
                        found.push(messages[i].intent);
                        groups.push({
                            intent: messages[i].intent,
                            text: messages.filter(function (msg) {
                                return msg.intent === messages[i].intent;
                            }).map(function (msg) { return msg.text })
                        });
                    }
                }

                return groups;
            }

            $scope.parseText = function(text)
            {
                return text.map(t => {
                    let text = t.text || t;

                    // parse XML like syntax
                    text = text.replace(/<.[^(><.)]+>/gm, (t) => '<span class="monospace">' + $sce.trustAsHtml(t.replace(/[\u00A0-\u9999<>\&]/gim, (i) => '&#' + i.charCodeAt(0) + ';')) + '</span>');
                    // parse ${} syntax
                    text = text.replace(/\${.*?}/gm, '<span class="monospace">$&</span>');

                    return text;
                }).join('<br>');
            }

            function _init() {
                var promise;

                if ($scope.blockId !== null && $scope.blockId !== undefined && $scope.blockId !== '') {
                    promise = ConvoworksApi.getBlockPreview($scope.service.service_id, $scope.blockId);
                } else {
                    promise = ConvoworksApi.getServicePreview($scope.service.service_id);
                }

                promise.then(function (preview) {
                    previewBlocks = preview.blocks.filter(b => {
                        return b.bot_says.length > 0 ||
                            b.user_says.length > 0 ||
                            b.bot_responds.length > 0;
                    });

                    $scope.filtered = angular.copy(previewBlocks);

                    $scope.$watch('search.query', function() {
                        if (!$scope.search.query || $scope.search.query === '') {
                            $scope.filtered = angular.copy(previewBlocks);
                        } else {
                            $scope.$applyAsync(() => {
                                $scope.filtered = previewBlocks.filter(b => b.block_name.toLowerCase().includes($scope.search.query.toLowerCase()));
                            });
                        }
                    });

                    $scope.ready = true;
                }, function (reason) {
                    $log.error('previewPanel could not get service preview, reason', reason);
                });
            }

            function _copyToClipboard(text) {
                // Create new element
                var el = document.createElement('textarea');
                // Set value (string to be copied)
                el.value = text;
                // Set non-editable to avoid focus and move outside of view
                el.setAttribute('readonly', '');
                el.style = {position: 'absolute', left: '-9999px'};
                document.body.appendChild(el);
                // Select text inside element
                el.select();
                // Copy text to clipboard
                document.execCommand('copy');
                // Remove temporary element
                document.body.removeChild(el);
            }
        }
    }
};
